@@ -10,7 +10,7 @@ use urpm::util;
 
 =head1 NAME
 
-urpm::install - Package installation transactin routines for urpmi
+urpm::install - Package installation transaction routines for urpmi
 
 =head1 SYNOPSIS
 
@@ -88,25 +88,38 @@ sub transaction_set_to_string {
 
 Standard logger for transactions
 
+See L<URPM> for parameters
+
 =cut
 
 # install logger callback
+my ($erase_logger, $index, $total_pkg, $uninst_count);
 sub install_logger {
-    my ($urpm, $type, $id, $subtype, $amount, $total) = @_;
-    my $pkg = defined $id && $urpm->{depslist}[$id];
-    my $total_pkg = $urpm->{nb_install};
+    my ($urpm, $type, undef, $subtype, $amount, $total) = @_;
     local $| = 1;
 
     if ($subtype eq 'start') {
 	$urpm->{logger_progress} = 0;
 	if ($type eq 'trans') {
+	    $total_pkg = $urpm->{nb_install};
 	    $urpm->{logger_count} ||= 0;
+	    $uninst_count = 0;
 	    my $p = N("Preparing...");
 	    print $p, " " x (33 - length $p);
 	} else {
-	    my $pname = $pkg ? $pkg->name : '';
-	    ++$urpm->{logger_count} if $pname;
-	    my $cnt = $pname ? $urpm->{logger_count} : '-';
+	    my $pname;
+	    my $cnt;
+	    if ($type eq 'uninst') {
+		$total_pkg = $urpm->{trans}->NElements - $index if !$uninst_count;
+		$cnt = ++$uninst_count;
+		$pname = N("removing %s", $urpm->{trans}->Element_fullname($index));
+		$erase_logger->($urpm, undef, undef, $subtype);
+	    } else {
+		# index already got bumped in {callback_open}:
+		$pname = $urpm->{trans}->Element_name($index-1);
+		++$urpm->{logger_count} if $pname;
+		$cnt = $pname ? $urpm->{logger_count} : '-';
+	    }
 	    my $s = sprintf("%9s: %-22s", $cnt . "/" . $total_pkg, $pname);
 	    print $s;
 	    $s =~ / $/ or printf "\n%9s  %-22s", '', '';
@@ -205,7 +218,8 @@ sub _apply_delta_rpm {
 }
 
 sub _schedule_packages {
-    my ($urpm, $trans, $install, $upgrade, $update, %options) = @_;
+    my ($urpm, $trans, $install, $upgrade, %options) = @_;
+    my $update = 0;
     my (@trans_pkgs, @produced_deltas);
     foreach my $mode ($install, $upgrade) {
 	foreach (keys %$mode) {
@@ -240,7 +254,7 @@ sub _schedule_packages {
 
 sub _get_callbacks {
     my ($urpm, $db, $trans, $options, $install, $upgrade, $have_pkgs) = @_;
-    my $index;
+    $index = 0;
     my $fh;
 
     my $is_test = $options->{test}; # fix circular reference
@@ -267,9 +281,10 @@ sub _get_callbacks {
 
     #- ensure perl does not create a circular reference below, otherwise all this won't be collected,
     #  and rpmdb won't be closed
-    my ($verbose, $callback_report_uninst) = ($options->{verbose}, $options->{callback_report_uninst});
-    $options->{callback_uninst} = sub { 	    
-	my ($_urpm, undef, undef, $subtype) = @_;
+    my $verbose = $options->{verbose};
+    $erase_logger = sub {
+	my ($urpm, undef, undef, $subtype) = @_;
+
 	if ($subtype eq 'start') {
 	    my ($name, $fullname) = ($trans->Element_name($index), $trans->Element_fullname($index));
 	    my @previous = map { $trans->Element_name($_) } 0 .. ($index - 1);
@@ -278,13 +293,20 @@ sub _get_callbacks {
 	    if (member($name, @previous)) {
 		$urpm->{log}("removing upgraded package $fullname");
 	    } else {
-		$callback_report_uninst and $callback_report_uninst->(N("Removing package %s", $fullname));
 		$urpm->{print}(N("removing package %s", $fullname)) if $verbose >= 0;
 	    }
 	    $index++;
 	}
     };
 
+    $options->{callback_uninst} ||= $options->{verbose} >= 0 ? \&install_logger : $erase_logger;
+
+    $options->{callback_error} ||= sub {
+	my ($urpm, undef, $id, $subtype) = @_;
+	my $n = $urpm->{depslist}[$id]->fullname;
+	$urpm->{error}("ERROR: '$subtype' failed for $n: ");
+    };
+
     if ($options->{verbose} >= 0 && $have_pkgs) {
 	$options->{callback_inst}  ||= \&install_logger;
 	$options->{callback_trans} ||= \&install_logger;
@@ -297,13 +319,17 @@ Install packages according to each hash (remove, install or upgrade).
 
 options: 
      test, excludepath, nodeps, noorder (unused), delta, 
-     callback_inst, callback_trans, callback_report_uninst,
+     callback_inst, callback_trans, callback_uninst,
+     callback_open_helper, callback_close_helper,
      post_clean_cache, verbose
   (more options for trans->run)
      excludedocs, nosize, noscripts, oldpackage, replacepkgs, justdb, ignorearch
 
+See L<URPM> for callback parameters
+
 =cut
 
+#- side-effects: uses a $urpm->{readmes}
 sub install {
     my ($urpm, $remove, $install, $upgrade, %options) = @_;
     $options{translate_message} = 1;
@@ -322,11 +348,11 @@ sub install {
 
     $trans->set_script_fd($options{script_fd}) if $options{script_fd};
 
-    my ($update, @errors) = 0;
+    my @errors;
 
     _schedule_packages_for_erasing($urpm, $trans, $remove);
 
-    my ($produced_deltas, @trans_pkgs) = _schedule_packages($urpm, $trans, $install, $upgrade, $update, %options);
+    my ($produced_deltas, @trans_pkgs) = _schedule_packages($urpm, $trans, $install, $upgrade, %options);
 
     if (!$options{nodeps} && (@errors = $trans->check(%options))) {
     } elsif (!$options{noorder} && (@errors = $trans->order)) {
@@ -335,7 +361,9 @@ sub install {
 
 	_get_callbacks($urpm, $db, $trans, \%options, $install, $upgrade, scalar @trans_pkgs);
 
+	local $urpm->{trans} = $trans;
 	@errors = $trans->run($urpm, %options);
+	delete $urpm->{trans};
 
 	#- don't clear cache if transaction failed. We might want to retry.
 	if (!@errors && !$options{test} && $options{post_clean_cache}) {
@@ -348,7 +376,7 @@ sub install {
 	    }
 	}
 
-	if ($options{verbose} >= 0) {
+	if ($options{verbose} >= 0 && !$options{justdb}) {
 	    foreach (keys %{$urpm->{readmes}}) {
 		$urpm->{print}("-" x 70 .  "\n" .
                                  N("More information on package %s", $urpm->{readmes}{$_}));
@@ -379,6 +407,6 @@ Copyright (C) 1999-2005 MandrakeSoft SA
 
 Copyright (C) 2005-2010 Mandriva SA
 
-Copyright (C) 2011-2012 Mageia
+Copyright (C) 2011-2013 Mageia
 
 =cut
