@@ -3,9 +3,9 @@ package urpm::media;
 # $Id: media.pm 270394 2010-07-30 00:48:07Z misc $
 
 use strict;
-use urpm qw(file_from_local_medium is_local_medium);
+use urpm 'file_from_local_medium', 'is_local_medium';
 use urpm::msg;
-use urpm::util qw(any append_to_file basename cat_ difference2 dirname intersection member output_safe begins_with copy_and_own file_size offset_pathname reduce_pathname);
+use urpm::util;
 use urpm::removable;
 use urpm::lock;
 use urpm::md5sum;
@@ -755,54 +755,6 @@ sub _auto_update_media {
     }
 }
 
-
-=item needed_extra_media($urpm)
-
-Return 2 booleans telling whether nonfree & tainted packages are installed respectively.
-
-=cut
-
-sub needed_extra_media {
-    my ($urpm) = @_;
-    my $db = urpm::db_open_or_die_($urpm);
-    my ($nonfree, $tainted);
-    $db->traverse(sub {
-	my ($pkg) = @_;
-	return if $nonfree && $tainted;
-	my $rel = $pkg->release;
-	$nonfree ||= $rel =~ /nonfree$/;
-	$tainted ||= $rel =~ /tainted$/;
-    });
-    ($nonfree, $tainted);
-}
-
-sub is_media_to_add_by_default {
-    my ($urpm, $distribconf, $medium, $product_id, $nonfree, $tainted) = @_;
-    my $add_by_default = !$distribconf->getvalue($medium, 'noauto');
-    my @media_types = split(':', $distribconf->getvalue($medium, 'media_types'));
-    return $add_by_default if !@media_types;
-    if ($product_id->{product} eq 'Free') {
-	if (member('non-free', @media_types)) {
-	    $urpm->{log}(N("ignoring non-free medium `%s'", $medium));
-	    $add_by_default = 0;
-	}
-    } else {
-	my $non_regular_medium = intersection(\@media_types, [ qw(backports debug source testing) ]);
-	if (!$add_by_default && !$non_regular_medium) {
-	    my $medium_name = $distribconf->getvalue($medium, 'name') || '';
-	    if ($medium_name =~ /Nonfree/ && $nonfree) {
-		$add_by_default = 1;
-		$urpm->{log}(N("un-ignoring non-free medium `%s' b/c nonfree packages are installed", $medium_name));
-	    }
-	    if ($medium_name =~ /Tainted/ && $tainted) {
-		$add_by_default = 1;
-		$urpm->{log}(N("un-ignoring tainted medium `%s' b/c tainted packages are installed", $medium_name));
-	    }
-	}
-    }
-    $add_by_default;
-}
-
 sub non_ignored_media {
     my ($urpm, $b_only_marked_update) = @_;
 
@@ -1139,7 +1091,6 @@ sub add_distrib_media {
 
     require urpm::mirrors;
     my $product_id = urpm::mirrors::parse_LDAP_namespace_structure(cat_('/etc/product.id'));
-    my ($nonfree, $tainted) = needed_extra_media($urpm);
 
     foreach my $media ($distribconf->listmedia) {
         my $media_name = $distribconf->getvalue($media, 'name') || '';
@@ -1157,8 +1108,14 @@ sub add_distrib_media {
 	    $is_update_media or next;
 	}
 
-        my $add_by_default = is_media_to_add_by_default($urpm, $distribconf, $media, $product_id, $nonfree, $tainted);
-
+        my $add_by_default = !$distribconf->getvalue($media, 'noauto');
+        my @media_types = split(':', $distribconf->getvalue($media, 'media_type'));
+        if ($product_id->{product} eq 'Free') {
+            if (member('non-free', @media_types)) {
+                $urpm->{log}(N("ignoring non-free medium `%s'", $media));
+                $add_by_default = 0;
+            }
+        }
 	my $ignore;
         if ($options{ask_media}) {
             $options{ask_media}->($media_name, $add_by_default) or next;
